'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = historicalCounts;


var _moment = require('moment');

var _moment2 = _interopRequireDefault(_moment);

var _d = require('d3');

var d3 = _interopRequireWildcard(_d);

var _overlayLine = require('./overlay-line');

var _overlayLine2 = _interopRequireDefault(_overlayLine);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var DAYS_PER_WEEK = 7,
    MINUTES_PER_HOUR = 60,
    SECONDS_PER_MINUTE = 60,
    MILLISECONDS_PER_SECOND = 1000;

var overlayDialogTopHeight = 43;
var overlayDialogTopWidth = 72;
var overlayDialogBottomHeight = 43;
var overlayDialogBottomWidth = 232;
var overlayDialogTextSize = 16; // Size of the text
var overlayDialogTextMargin = 12; // Y spacing between text and its container
var overlayDialogBorderRadius = 4;
var overlayDialogTopIconCenterOffset = 21;
var overlayDialogTopTextCenterOffset = 12;

var overlayDialogTopBottomMargin = 10;
var overlayDialogBottomTopMargin = 48;

var leftMargin = 60;
var rightMargin = 5;
var bottomMargin = overlayDialogBottomTopMargin + overlayDialogTopBottomMargin + 42;
var topMargin = 16 + overlayDialogTopHeight + overlayDialogTopBottomMargin;

// Distance between the overlay dialog and the line on the graph
var overlayDialogDistanceToLine = 10;

// Distance the overlay dialog has to get to the right side of the graph before the dialog breaks to
// the other side of the line.
var overlayDialogBreakToLeftPadding = 20;

// The count graph component
function historicalCounts(elem) {
  var svg = d3.select(elem).append('svg').attr('class', 'historical-counts');

  // Add filter for use in making shadows behind things.
  svg.append('filter').attr('id', 'shadow-blur').append('feGaussianBlur').attr('stdDeviation', 5);

  var svgGroup = svg.append('g').attr('transform', 'translate(' + leftMargin + ',' + topMargin + ')');

  // The capacity background overlay is drawn at the bottom of the stack in here.
  var capacityGroup = svgGroup.append('g').attr('class', 'capacity-group');

  // The graph path that shows the data goes in here.
  var graphGroup = svgGroup.append('g').attr('class', 'graph-group');

  // The axes go in here.
  var axisGroup = svgGroup.append('g').attr('class', 'axis-group');

  // Flags are all stuck in this group, must be above the axes, otehrwise the horizontal axis rules
  // cross over the flag line and make it look like a dotted line.
  var flagGroup = svgGroup.append('g').attr('class', 'flag-group');

  // Put all the overlay stuff in here (the line and dialog)
  var overlayGroup = svgGroup.append('g').attr('class', 'overlay-group');

  // The hidden rectangle to use to detect mouse position
  var overlayRect = svgGroup.append('g').attr('class', 'overlay-rect');

  return function (_ref) {
    var start = _ref.start,
        end = _ref.end,
        width = _ref.width,
        height = _ref.height,
        data = _ref.data,
        capacity = _ref.capacity,
        initialCount = _ref.initialCount,
        timeZoneLabel = _ref.timeZoneLabel,
        timeZoneOffset = _ref.timeZoneOffset;

    width = width || 800;
    height = height || 400;
    capacity = capacity || null;
    initialCount = initialCount || 0;

    // Convert the timestamp in each item into epoch milliseconds.
    data = (data || []).map(function (i) {
      if (i.timestamp instanceof _moment2.default) {
        return Object.assign({}, i, { timestamp: i.timestamp.valueOf() });
      } else {
        return Object.assign({}, i, { timestamp: _moment2.default.utc(i.timestamp).valueOf() });
      }
    });

    var flags = [];

    // Adjust the svg size and viewbox to match the passed in values.
    svg.attr('height', height).attr('width', width).attr('viewBox', '0 0 ' + width + ' ' + height);

    if (!Array.isArray(data)) {
      throw new Error('A \'data\' prop is required.');
    }

    // Get the drawn graph size, minus the borders.
    var graphWidth = width - leftMargin - rightMargin;
    var graphHeight = height - topMargin - bottomMargin;

    // Get first and last timestamps
    var firstEvent = data.length && data[0];
    var dataStart = firstEvent ? firstEvent.timestamp : _moment2.default.utc();
    var domainStart = start || dataStart;

    var lastEvent = data.length && data[data.length - 1];
    var lastCount = lastEvent ? lastEvent.count : 0;
    var dataEnd = lastEvent ? lastEvent.timestamp : _moment2.default.utc();
    var domainEnd = end || dataEnd;

    // Construct scales
    var xScale = d3.scaleLinear().rangeRound([graphWidth, 0]).domain([domainEnd, domainStart]);
    var largestCount = data.length > 0 ? Math.max.apply(Math, data.map(function (i) {
      return Math.max(i.count, initialCount);
    })) : 0;
    var smallestCount = data.length > 0 ? Math.min.apply(Math, data.map(function (i) {
      return Math.min(i.count, initialCount);
    })) : 0;
    var yScale = d3.scaleLinear().rangeRound([graphHeight - 10, 0]).domain([smallestCount, largestCount]);

    var lastX = xScale(_moment2.default.min(domainEnd, _moment2.default.utc()));
    var lastY = yScale(lastEvent.count);

    // Render capacity overlay behind the graph path, if a capacity was passed.
    if (capacity) {
      var capacityHeightInPx = yScale(capacity);
      var capacitySelection = capacityGroup.selectAll('.historical-counts-capacity-region').data([capacityHeightInPx]);

      capacitySelection.enter().append('rect').attr('class', 'historical-counts-capacity-region').merge(capacitySelection).attr('x', 0).attr('y', function (d) {
        return d;
      }).attr('width', graphWidth).attr('height', function (d) {
        return graphHeight - d;
      });

      capacitySelection.exit();
    }

    // Generate the svg path for the graph line.
    var pathPrefix = 'M1,' + graphHeight + ('L1,' + yScale(initialCount)) + ('H' + xScale(dataStart));
    var pathSuffix = 'H' + lastX + 'V' + graphHeight + 'H1';

    // Build the path by looping through the data
    var linePath = data.reduce(function (total, i) {
      // Extract flags so we can draw them on top
      if (i.flag) {
        flags.push(i);
      }

      // Step to the new point
      var xPosition = xScale(i.timestamp);
      var yPosition = yScale(i.count);
      if (xPosition >= 0) {
        return total + 'H' + xPosition + 'V' + yPosition;
      } else {
        return total;
      }
    }, '');

    var graphSelection = graphGroup.selectAll('.historical-counts-path').data([data]);

    var graphEnterSelection = graphSelection.enter();
    graphEnterSelection.append('path').attr('class', 'historical-counts-path');
    graphEnterSelection.append('path').attr('class', 'historical-counts-path-outline');

    var graphMergeSelection = graphEnterSelection.merge(graphSelection);
    graphMergeSelection.select('.historical-counts-path').attr('d', function (d) {
      return pathPrefix + linePath + pathSuffix;
    });
    graphMergeSelection.select('.historical-counts-path-outline').attr('d', function (d) {
      return pathPrefix + linePath;
    });

    graphSelection.exit();

    // Draw the axes in the svg

    // Remove all axes that are already drawn
    axisGroup.selectAll("g").remove();

    // Render the y axis.
    axisGroup.call(require('./axis-y').default, yScale, graphWidth, smallestCount, largestCount, capacity);

    var totalDuration = domainEnd - domainStart;
    var xAxis = d3.axisBottom(xScale).ticks(Math.floor(totalDuration / (MINUTES_PER_HOUR * SECONDS_PER_MINUTE * MILLISECONDS_PER_SECOND))).tickSizeOuter(0).tickFormat(function (d, i) {
      var timeFormat = (0, _moment2.default)(d).add(timeZoneOffset, 'hours').format('hA');
      return timeFormat.slice(0, timeFormat.startsWith('12') ? -1 : -2).toLowerCase();
    });
    axisGroup.append("g").attr("class", 'historical-counts-axis-x').attr("transform", 'translate(0,' + graphHeight + ')').call(xAxis);

    // Generate flag lines
    // Each consists of a `g.flag` wrapper, with a `path.flag-line` and `text.flag-line-label`
    // inside.
    var flagSelection = flagGroup.selectAll('.flag').data(flags);

    var flagEnterSelection = flagSelection.enter().append('g').attr('class', 'flag');
    flagEnterSelection.append('line').attr('class', 'flag-line').attr('x1', 0).attr('y1', 0).attr('x2', 0).attr('y2', graphHeight);
    flagEnterSelection.append('rect').attr('class', 'flag-line-label-background').attr('x', 0).attr('y', 0).attr('width', 40).attr('height', 28);
    flagEnterSelection.append('text').attr('class', 'flag-line-label');

    var flagMergeSelection = flagEnterSelection.merge(flagSelection);
    // Move the group / line / text to the flag's position
    flagMergeSelection.attr('transform', function (d) {
      return 'translate(' + xScale(d.timestamp) + ',0)';
    });

    // Adjust if the graph height changed
    flagMergeSelection.select('.flag-line').attr('y2', graphHeight);

    // Adjust the flag label
    flagMergeSelection.select('.flag-line-label').text(function (d) {
      return d.count;
    });

    flagSelection.exit().remove('.flag');

    // Draw the overlay line if there is any data
    if (data.length) {
      var showOverlay = function showOverlay() {
        var mouseX = d3.mouse(overlayRect.node())[0];
        overlayGroup.call(_overlayLine2.default, xScale, yScale, domainStart, domainEnd, graphWidth, graphHeight, initialCount, timeZoneLabel, timeZoneOffset, overlayDialogTopBottomMargin, overlayDialogBottomTopMargin, overlayDialogBorderRadius, overlayDialogTopWidth, overlayDialogTopHeight, overlayDialogBottomWidth, overlayDialogBottomHeight, overlayDialogTopIconCenterOffset, overlayDialogTopTextCenterOffset, data, mouseX);
      };

      overlayRect.append('rect').attr('x', -1 * leftMargin).attr('y', -1 * topMargin).attr('width', leftMargin + graphWidth + rightMargin).attr('height', topMargin + graphHeight + bottomMargin).attr('fill', 'transparent').on('touchstart', showOverlay).on('touchmove', showOverlay).on('mousemove', showOverlay).on('mouseout', function () {
        overlayGroup.call(_overlayLine2.default, xScale, yScale, domainStart, domainEnd, graphWidth, graphHeight, initialCount, timeZoneLabel, timeZoneOffset, overlayDialogTopBottomMargin, overlayDialogBottomTopMargin, overlayDialogBorderRadius, overlayDialogTopWidth, overlayDialogTopHeight, overlayDialogBottomWidth, overlayDialogBottomHeight, overlayDialogTopIconCenterOffset, overlayDialogTopTextCenterOffset, data, null);
      });
    }
  };
}

